unit UMainForm;

{$mode Delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, SynEdit, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, StdCtrls, ComCtrls, UBrainFuck;

type

  { TMainForm }

  TMainForm = class(TForm)
    ACompile: TAction;
    AAbout: TAction;
    AHelp: TAction;
    ARun: TAction;
    AProjectNew: TAction;
    AExit: TAction;
    AProjectSaveAs: TAction;
    AProjectClose: TAction;
    AProjectSave: TAction;
    AProjectOpen: TAction;
    ActionList1: TActionList;
    ImageList1: TImageList;
    MainMenu1: TMainMenu;
    MemoSource: TMemo;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    StatusBar1: TStatusBar;
    TabControl1: TTabControl;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    procedure AExitExecute(Sender: TObject);
    procedure AProjectCloseExecute(Sender: TObject);
    procedure AProjectNewExecute(Sender: TObject);
    procedure AProjectOpenExecute(Sender: TObject);
    procedure AProjectSaveAsExecute(Sender: TObject);
    procedure AProjectSaveExecute(Sender: TObject);
    procedure ARunExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MemoSourceChange(Sender: TObject);
  private
    { private declarations }
  public
    Modified: Boolean;
    ProjectFileName: string;
    BrainFuckInterpreter: TBrainFuckInterpreter;
    procedure UpdateInterface;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
  UInterpreterForm, UApplicationInfo;

{ TMainForm }

procedure TMainForm.FormShow(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TMainForm.MemoSourceChange(Sender: TObject);
begin
  Modified := True;
  UpdateInterface;
end;

procedure TMainForm.UpdateInterface;
var
  Title: string;
begin
  AProjectSave.Enabled := Modified;
  Title := ApplicationInfo.Name;
  if ProjectFileName <> '' then Title := ProjectFileName + ' - ' + Title;
  if Modified then Title := Title + ' *';
  Caption := Title;
  MemoSource.Enabled := ProjectFileName <> '';
  AProjectClose.Enabled := ProjectFileName <> '';
  ARun.Enabled := ProjectFileName <> '';
  ACompile.Enabled := ProjectFileName <> '';
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  BrainFuckInterpreter := TBrainFuckInterpreter.Create;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  BrainFuckInterpreter.Free;
end;

procedure TMainForm.ARunExecute(Sender: TObject);
begin
  InterpreterForm.Show;
  BrainFuckInterpreter.Input := InterpreterForm.MemoInput.Lines.Text;
  BrainFuckInterpreter.Source := MemoSource.Lines.Text;
  BrainFuckInterpreter.Run;
  InterpreterForm.MemoOutput.Lines.Text := BrainFuckInterpreter.Output;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
end;

procedure TMainForm.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AProjectCloseExecute(Sender: TObject);
begin
  Modified := False;
  ProjectFileName := '';
  MemoSource.Clear;
  UpdateInterface;
end;

procedure TMainForm.AProjectNewExecute(Sender: TObject);
begin
  ProjectFileName := 'Project.bf';
  MemoSource.Clear;
  UpdateInterface;
end;

procedure TMainForm.AProjectOpenExecute(Sender: TObject);
begin
  OpenDialog1.FileName := ProjectFileName;
  if OpenDialog1.Execute then begin
    MemoSource.Lines.LoadFromFile(OpenDialog1.FileName);
    ProjectFileName := OpenDialog1.FileName;
    UpdateInterface;
  end;
end;

procedure TMainForm.AProjectSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.FileName := ProjectFileName;
  if SaveDialog1.Execute then begin
    MemoSource.Lines.SaveToFile(SaveDialog1.FileName);
    ProjectFileName := SaveDialog1.FileName;
    Modified := False;
    UpdateInterface;
  end;
end;

procedure TMainForm.AProjectSaveExecute(Sender: TObject);
begin
  if Modified then AProjectSaveAs.Execute;
end;

end.

